/*
 * fast_graph_geodesics.cc
 * =======================
 *
 * Implement a fast approximation algorithm for graph geodesics
 *
 * History
 * -------
 * :2010-05-21: Initial coding (and it works!)
 * :2010-05-26: Added path_histogram option, divided into header file
 *   for Matlab implementation
 */

#include <assert.h>
#include <vector>
#include <math.h>
#include <string.h>

#include "sparfun_util.h"

#include "fast_graph_geodesics.h"

void print_sets(const byte* set, vertex nverts, size_t nbytes, size_t nrep) {
    for (vertex i=0; i<nverts; i++) {
        for (size_t j=0; j<nrep; j++) {
            for (size_t k=0; k<nbytes; k++) {
                int x = set[i*nbytes*nrep+j*nbytes+k];
                static char b[8] = {0};
                for (int z=128,y=0; z>0; z>>=1,y++) {
                    b[y] = ( ((x & z) == z) ? 49 : 48);
                }
                printf("%8s-", b);
            }
            printf("  ");
        }
        printf("\n");
    }
}

/** Return the index of the first bit
 * 
 * Use the following Matlab code for the bit table:
 * for i=0:255, for b=1:8, 
 *   if bitget(i,b)==1, fprintf('%i, ', b-1); break; end, 
 * end, end, fprintf('\n');
 */
int set_first_nonzero_bit(byte* bytes, size_t nbytes) {
    int rval = -1;
    static int bit_table[256] = {
        -1, // 0
        0, // 1
        1, 0, // 2, 3
        2, 0, 1, 0, //  4, 5, 6, 7
        3, 0, 1, 0, 2, 0, 1, 0, // 8-15
        4, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, // 16-31
        5, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, // 32-63
           4, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, 
        6, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, // 64-127
           4, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, 
           5, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, 
           4, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, 
        7, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, // 128-255
           4, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, 
           5, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, 
           4, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, 
           6, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, 
           4, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, 
           5, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, 
           4, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, 
    };
    
    for (size_t i=0; i<nbytes; i++) {
        int bit = bit_table[bytes[i]];
        if (rval >= 0) {
            bytes[i] = 0;
        } else if (bit >= 0) {
            bytes[i] = 1<<bit;
            rval = i*8 + bit;
        } else {
            assert(bytes[i] == 0);
        }
            
    }

    return rval;
}

/** Return the index of the first bit
 * 
 * Use the following Matlab code for the bit table:
 * for i=0:255, for b=1:8, 
 *   if bitget(i,b)==1, fprintf('%i, ', b-1); break; end, 
 * end, end, fprintf('\n');
 */
int get_first_zero_bit(byte* bytes, size_t nbytes) {
    int rval = -1;
    static int bit_table[256] = {
        0, // 0
        1, 0, // 1, 2
        2, 0, 1, 0, // 3, 4, 5, 6
        3, 0, 1, 0, 2, 0, 1, 0, // 7-14
        4, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, // 15-30
        5, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, // 31-62
           4, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, 
        6, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, // 63-126
           4, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, 
           5, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, 
           4, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, 
        7, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, // 127-254
           4, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, 
           5, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, 
           4, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, 
           6, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, 
           4, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, 
           5, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, 
           4, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, 
         -1 // 255

    };
    
    for (size_t i=0; i<nbytes; i++) {
        int bit = bit_table[bytes[i]];
        if (bit >= 0) {
            rval = i*8 + bit;
            break;
        }   
    }

    return rval;
}

/** Compute the estimated size from a set of Flajoulet-Martin bitstrings
 * 
 * @param allbitstrings the raw data for the bitstrings, nbytes*nrep 
 *   bytes and each group of nbytes corresponds to a single bitstring.
 * @param nbytes the number of bytes in each bitstring
 * @param nrep the number of bitstrings
 * 
 * Let Xi be the first 0 bit set in each bitstring.  Then the estimate 
 * of the number of unique items is
 *   2^{1/nrep * sum(Xi)}/0.77351
 * 
 * @return if all bitstrings corresponds to a Flajoules-Martin string,
 *   then, the estimate should approximate the number of unique items.
 */
double fm_bitstring_estimate(byte* allbitstrings, size_t nbytes, size_t nrep)
{
    // first compute the first average 0 bit
    double mean_bit = 0;
    for (size_t i=0; i<nrep; i++) {
        int bit = get_first_zero_bit(&allbitstrings[nbytes*i], nbytes);
        mean_bit += (double)bit;
    }
    mean_bit = mean_bit /(double)nrep;
    // now estimate the size
    double rval = pow(2.0,mean_bit);
    rval = rval/0.77531;
    
    return rval;
}

void bitwise_or(byte* x, const byte* y, size_t nbytes) {
    size_t nints = nbytes/4;
    size_t nextra = nbytes%4;
    unsigned int *ix = (unsigned int*)x;
    const unsigned int *iy = (const unsigned int*)y;
    for (size_t i=0; i<nints; i++) {
        ix[i] |= iy[i];
    }
    for (size_t i=0; i<nextra; i++) {
        x[i+sizeof(unsigned int)*nints] |= y[i+sizeof(unsigned int)*nints];
    }
}

void random_bytes(byte* x, size_t nbytes) {
    size_t nints = nbytes/4;
    size_t nextra = nbytes%4;
    unsigned int *ix = (unsigned int*)x;
    for (size_t i=0; i<nints; i++) {
        ix[i] = sf_rand_uint();
    }
    for (size_t i=0; i<nextra; i++) {
        x[i+sizeof(unsigned int)*nints] = sf_randbyte();
    }
}


/*
 * @param nverts the number of vertices (must be a vertex type itself)
 * @param nedges the number of edges (size_t because it could be larger 
 *  than the number of edges
 * @param edges a list of vertices that corresponds to edge pairs
 *   (edges[0], edges[1]) is the first edge 
 *   (edges[2], edges[3]) is an edge
 *   ...
 *   (edges[2*nedges-2], edges[2*nedges-1]) is the last edge
 * @param nbytes the number of bytes to use in each hash
 * @param nrep the number of hashes to use
 * @param maxiter the maximum number of iterations 
 *   if this parameter is 0, then use the number of vertices from the 
 *   graph
 * @param verbose a FILE* stream to output verbose data 
 * @param largest_geodesics an array to hold the longest shortest 
 *   path, if this is null then we don't estimate these distances.
 * @param npaths an array of size n to hold 
 *   an estimate of the number of shortest paths of 
 *   length [0,n-1] where n is the number of vertices.  If the
 *   variable is NULL, then these estimates are not computed.
 * @param diameter a pointer to a location for the output diameter
 *   if this is null, then the diameter isn't returned (you could still
 *   estimate it from longest_geodesics if ou requested that...
 */ 
void fast_geodesics_estimate(
        vertex nverts,
        size_t nedges,
        vertex* edges,
        size_t nbytes,
        size_t nrep,
        size_t maxiter,
        FILE* verbose, 
        vertex* largest_geodesics,
        double* npaths,
        vertex* diameter)
{
    // start wall timer
    wall_timer t0;
    
    // set maxiter reasonably.
    if (maxiter == 0) {
        maxiter = nverts;
    }
    
    // allocate memory
    std::vector<byte> hashes(nbytes*nverts*nrep);
    std::vector<byte> reach_sets(nbytes*nverts*nrep);
    std::vector<byte> old_reach_sets(nbytes*nverts*nrep);
    std::vector<double> old_reach_size(0); // initialize to empty
    
    if (largest_geodesics) {
        memset(largest_geodesics, 0, sizeof(vertex)*nverts);
    }
    
    if (npaths) {
        if (verbose) {
            fprintf(verbose, "computing path histogram too\n");
        }
        memset(npaths, 0, sizeof(double)*nverts);
        old_reach_size.resize(nverts); // resize if we'll use it.
        
        // initialize with every vertex reachable from itself.
        for (size_t i=0; i<nverts; i++) {
            old_reach_size[i] = 1.;
        }
        npaths[0] = (double)nverts;
    }
    
    // generate random data
    random_bytes(&hashes[0], nbytes*nverts*nrep);
    // truncate to the first non-zero bit only
    for (size_t i=0; i<nverts*nrep; i++) {
        set_first_nonzero_bit(&hashes[i*nbytes], nbytes);
    }
    
    
    // start the iteration
    reach_sets = hashes;
        
    size_t last_iter = 0;
    
    for (size_t iter = 0; iter < maxiter; ++iter) {
        old_reach_sets = reach_sets;
        reach_sets = hashes;
        for (size_t j=0; j<nedges; j++) {
            vertex src = edges[2*j];
            vertex dst = edges[2*j+1];
            bitwise_or(
                &reach_sets[src*nbytes*nrep],
                &old_reach_sets[dst*nbytes*nrep],
                nbytes*nrep);
        }
        
        //print_sets(&reach_sets[0], nverts, nbytes, nrep);
        
        // check for completion
        vertex num_vertices_done = 0;
        double vertices_at_dist_i = 0.;
        for (vertex i=0; i<nverts; i++) {
            bool vertex_unchanged = true;
            byte* oldval = &old_reach_sets[i*nbytes*nrep];
            byte* newval = &reach_sets[i*nbytes*nrep];
            for (size_t bytei = 0; bytei < nbytes*nrep; ++bytei) {
                if (oldval[bytei] != newval[bytei]) {
                    vertex_unchanged = false;
                    break;
                }
            }
            if (vertex_unchanged) {
                // the vertex didn't change, so we have one more done
                num_vertices_done += 1;
                if (largest_geodesics) {
                    if (largest_geodesics[i] == 0) {
                        largest_geodesics[i] = iter+1;
                    }
                }
            } else {
                // the vertex changed its value
                if (largest_geodesics) {
                    if (largest_geodesics[i] > 0) {
                        // this vertex isn't set now. 
                        largest_geodesics[i] = 0;
                    }
                }
            }
            if (npaths) {
                // estimate the number of vertices reachable 
                // at dist "iter+1", to do this, we have to use 
                // the properties of the FM count

                if (vertex_unchanged) {
                    // we can skip this vertex as we won't compute
                    // anything different 
                } else {
                    // the vertex did change, so we need to update
                    // our counts
                    double reachable_est = fm_bitstring_estimate(
                                            &reach_sets[i*nbytes*nrep], 
                                            nbytes, nrep);
                                            
                    double new_vertices = reachable_est - old_reach_size[i];
                    
                    old_reach_size[i] = reachable_est;
                    vertices_at_dist_i += new_vertices;
                }
            }
        }
        
        if (verbose) {
            fprintf(verbose, 
                "iter: %4Zi  set: %10i (%3i%%)  reach: %10.0f  dt:%6.1fs \n",
                iter+1, num_vertices_done, 
                (int)(100.*(double)num_vertices_done/(double)nverts), 
                vertices_at_dist_i,t0.dt());
        }
        
        if (npaths) {                
            npaths[iter+1] = vertices_at_dist_i;
        }
        
        if (num_vertices_done == nverts) {
            last_iter = iter;
            break;
        }
    }
    
    if (largest_geodesics) {
        for (vertex i=0; i<nverts; i++) {
            largest_geodesics[i] -= 1;
        }
    }
    
    if (diameter) {
        *diameter = last_iter-1; 
    }
}

             




