function [r path_hist] = fast_graph_geodesics(A,varargin)
% FAST_GRAPH_GEODESICS Estimate the longest shortest path from each node
%
% r = fast_graph_geodesics(A) computes an estimate of the longest shortest path
% from each node in the graph using a probabilistic set union counting
% operation.  The fundamental algorithm is the Flajoules-Martin count, and
% the adaptation to computing the longest shortest path is from U. Kang et
% al. [SDM 2010].
%
% r = graph_radius(A,'nbytes',k,'nrep',m) controls the number of bits used
% in the hash-codes k, and the number of independent hashes m.  The
% defaults of k=8 and m=32 should suffice for graphs up to a few billion
% vertices.  Increase k for larger graphs; increase m for higher accuracy
% (though m improves accuracy as 1/sqrt(m), so don't expect too much.)
%
% This function ignores the non-zero values and only uses the non-zero
% structure of the matrix A.  It works for directed and undirected graphs.
%
% Example:
%   N = 50;
%   A = spones(delsq(numgrid('S',N+2))); 
%   r = fast_graph_geodesics(A);  
%   B = A + sprand(N*N,N*N,0/(N*N)); B=B|B';
%   r_short = fast_graph_geodesics(B);
%   [n1,x1] = hist(r); [n2,x2] = hist(r_short);
%   plot(x1,n1,'b.-',x2,n2,'r.-'); legend('Mesh','Mesh + Shortcuts');

% David F. Gleich
% Copyright, 2010

args = struct(varargin{:});
if isfield(args,'nbytes'), k=args.nbytes; else k=8; end
if isfield(args,'nrep'), m=args.nrep; else m=32; end
if isfield(args,'verbose'), verbose=args.verbose; else verbose=false; end

n = size(A,1);
maxiter = n;

% convert to edge data
[i,j] = find(A);
edges = uint32([i-1,j-1]');

[r, path_hist] = fast_graph_geodesics_mex( ...
    uint32(n), edges,uint32(k), uint32(m), uint32(verbose));
r = double(r); % the function returns uint32s
