/*
 * fastgeo.cc
 * =======================
 *
 * Implement a fast approximation algorithm for graph geodesics
 *
 * History
 * -------
 * :2010-05-26: Initial coding -- based on fast_graph_geodesics.cc
 */

#include <assert.h>
#include <vector>
#include <math.h>
#include <iostream>
#include <stdio.h>
#include <string.h>

#include <zlib.h>
 
#include "sparfun_util.h"

#include "fast_graph_geodesics.h"


int load_edges(
    const char* filename,
    vertex* nverts_out,
    std::vector<vertex>& edges) 
{
    FILE *f = fopen(filename, "rt");
    if (!f) {
        return -1;
    }
    
    vertex nverts, nverts_dup;
    size_t nedges;
    
    fscanf(f, "%i %i %zu", &nverts, &nverts_dup, &nedges);
    
    *nverts_out = nverts;
    
    edges.resize(nedges*2);
    
    for (size_t i = 0; i<nedges; i++) {
        vertex src, dst;
        double weight;
        fscanf(f, "%i %i %lf", &src, &dst, &weight);
        edges[2*i] = src;
        edges[2*i+1] = dst;
    }
    
    fclose(f);
    
    return (0);
}

int load_edges_bsmat_gz(
    const char* filename,
    vertex* nverts_out,
    std::vector<vertex>& edges) 
{
    gzFile f = gzopen(filename, "r");
    if (!f) {
        return -1;
    }
    
    unsigned int uint32var;
    
    vertex nverts, nverts_dup;
    size_t nedges;
    
    
    
    gzread(f, &uint32var, sizeof(unsigned int)); nverts = uint32var;
    gzread(f, &uint32var, sizeof(unsigned int)); nverts_dup = uint32var;
    gzread(f, &uint32var, sizeof(unsigned int)); nedges = uint32var;

    *nverts_out = nverts;
    
    edges.resize(nedges*2);
    
    for (size_t i = 0; i<nedges; i++) {
        vertex src, dst;
        double weight;
        gzread(f, &uint32var, sizeof(unsigned int)); src = uint32var;
        gzread(f, &uint32var, sizeof(unsigned int)); dst = uint32var;
        gzread(f, &weight, sizeof(double)); 
        edges[2*i] = src;
        edges[2*i+1] = dst;
    }
    
    gzclose(f);
    
    return (0);
}

int main(int argc, char **argv) {
	// timing
	wall_timer t0;
    // option parsing
    bool verbose = true;
    long long seed = 0;
    int nbytes = 8;
    int nrep = 53;
    
    // todo get options

    // get options
    seed = sf_timeseed();
    
    const char* filename = argv[1];
    
    std::vector<vertex> edges;
    vertex nverts;
    
    if (verbose) {
		std::cerr << "edgefile = " << filename << std::endl;
		std::cerr << "nbytes = " << nbytes << std::endl;
		std::cerr << "nrep = " << nrep << std::endl;
		std::cerr << "seed = " << seed << std::endl;
	}
    
    // very simple test for the extension
	t0.start();
    if (strstr(filename,".bsmat.gz") != NULL) {
		if (verbose) { 
			std::cerr << "reading " << filename 
					<< " as bsmat.gz" << std::endl;
		}
		load_edges_bsmat_gz(filename,&nverts,edges);
		if (verbose) {  
			std::cerr << "done! (" << t0.dt() << " seconds)" << std::endl;
		}
    } else {
		if (verbose) { 
			std::cerr << "reading " << filename 
					<< " as smat" << std::endl;
		}
        load_edges(filename,&nverts,edges);
        if (verbose) {  
			std::cerr << "done! (" << t0.dt() << " seconds)" << std::endl;
		}
    }
    
    if (verbose) {
		std::cerr << "graph: " << nverts << " vertices, " 
			<< edges.size()/2 << " edges" << std::endl;
	}
    
    vertex diameter;
    std::vector<vertex> geodesics(nverts,0);
    std::vector<double> path_hist(nverts,0);
    
    FILE* verbose_stream = NULL;
    if (verbose) {
        verbose_stream = stderr;
    }
    
    fast_geodesics_estimate(nverts, edges.size()/2, &edges[0],
        nbytes,  nrep,  0,  verbose_stream,
        &geodesics[0],  &path_hist[0] ,  &diameter);

    for (vertex i=0; i<nverts; i++) {
        std::cout << geodesics[i] << std::endl;
    }
    
    for (vertex i=0; i<nverts; i++) {
        if (path_hist[i] > 0) {
            std::cout << i << " " << path_hist[i] << std::endl;
        }
    }
    
    if (verbose) {
		std::cerr << "diameter: " << diameter << std::endl;
		std::cerr << "total time: " << t0.dt() << " seconds" << std::endl;
	}

    return (0);
}
