%% Evaluate fast_graph_geodesics approximation
% In this script, we evaluate the fast graph geodesics code against the
% exact answer computed using matlab_bgl.  We do this for a series of
% structured graphs: 3-5d grids, cycles, and erdos-reyni graphs
% also, we do it for some of the largest test graphs in MatlabBGL.

%% Setup
% add paths to get the
mbgldir = '~/dev/matlab-bgl/4.0';
addpath(mbgldir);
addpath('~/dev/matlab-extern/export_fig');

set(0,'defaultlinelinewidth',1.4);
set(0,'defaultaxesfontsize',14);
set(0,'defaultaxeslinewidth',0.9);

%% Case 1: Structured graphs

%% 
% 3-d grids
A = grid_graph([20,20,20]);
results = eval_graph_geodesics(A);
%%
plot_geodesics_evaluation(results)
title(sprintf('3d grid: %i verts, %i edges',size(A,1),nnz(A)));
set(gca,'Color','none'); set(gcf,'Color','none');
export_fig 'grid-3d-20.png' -m2
%%
plot_geodesics(results);
export_fig 'grid-3d-20-geodesics.png' -m2

%%
% 4-d grids
A = grid_graph([10,10,10,10]);
results = eval_graph_geodesics(A);
%%
plot_geodesics_evaluation(results)
title(sprintf('4d grid: %i verts, %i edges',size(A,1),nnz(A)));
set(gca,'Color','none'); set(gcf,'Color','none');
export_fig 'grid-4d-10.png' -m2

%%
% 5-d grids
A = grid_graph([6,6,6,6,6]);
results = eval_graph_geodesics(A);
%%
plot_geodesics_evaluation(results)
title(sprintf('5d grid: %i verts, %i edges',size(A,1),nnz(A)));
set(gca,'Color','none'); set(gcf,'Color','none');
export_fig 'grid-5d-6.png' -m2

%%
% directed cycle graphs (worst case for approximation)
N = 500;
A = cycle_graph(N,'directed',1);
results = eval_graph_geodesics(A);
%%
plot_geodesics_evaluation(results)
title(sprintf('cycle: %i verts, %i edges',size(A,1),nnz(A)));
set(gca,'Color','none'); set(gcf,'Color','none');
export_fig 'cycle-500.png' -m2

%%
% big directed cycle
N = 2500;
A = cycle_graph(N,'directed',1);
results = eval_graph_geodesics(A);
%%
plot_geodesics_evaluation(results)
title(sprintf('directed cycle: %i verts, %i edges',size(A,1),nnz(A)));

%% Case 2: random graphs
% erdos-reyni
N = 2500; d=5;
A = erdos_reyni(N,d/N);
results = eval_graph_geodesics(A);
%%
plot_geodesics_evaluation(results)
title(sprintf('erdos-reyni (d=%i) : %i verts, %i edges',d, size(A,1),nnz(A)));
set(gca,'Color','none'); set(gcf,'Color','none');
export_fig 'er-2500-5.png' -m2
%%
plot_geodesics(results);
export_fig 'er-2500-5-geodesics.png' -m2


%% Case 3: Real-world graphs

%% 
% Use minnesota
load(fullfile(mbgldir,'graphs','minnesota.mat'));
results = eval_graph_geodesics(A);
%%
plot_geodesics_evaluation(results)
title(sprintf('minnesota road network: %i verts, %i edges',size(A,1),nnz(A)));
set(gca,'Color','none'); set(gcf,'Color','none');
export_fig 'minnesota.png' -m2

%%
plot_geodesics(results);
export_fig 'minnesota-geodesics.png' -m2

%% 
% Use wb-cs.stanford
load(fullfile(mbgldir,'graphs','cs-stanford.mat'));
results = eval_graph_geodesics(A);
%%
plot_geodesics_evaluation(results)
title(sprintf('stanford web-graph: %i verts, %i edges',size(A,1),nnz(A)));
set(gca,'Color','none'); set(gcf,'Color','none');
export_fig 'cs-stanford.png' -m2
%%
plot_geodesics(results);
export_fig 'cs-stanford-geodesics.png' -m2

%% 
% Use tapir
load(fullfile(mbgldir,'graphs','tapir.mat'));
results = eval_graph_geodesics(A);
plot_geodesics_evaluation(results)
title(sprintf('tapir 2d-mesh: %i verts, %i edges',size(A,1),nnz(A)));
set(gca,'Color','none'); set(gcf,'Color','none');
export_fig 'tapir.png' -m2

%% Case 4: Big graphs

%% 
% use lcsh because of the double bump
load('~/data/lcsh/lcsh.matlab.mat');
A = LCSH;
dt=tic
results = eval_graph_geodesics(A);
toc(dt)
save 'lcsh-eval-results.mat' results

%%
load('lcsh-eval-results.mat');
%%
plot_geodesics_evaluation(results)
title(sprintf('lcsh: %i verts, %i edges',size(A,1),nnz(A)));
set(gca,'Color','none'); set(gcf,'Color','none');
export_fig 'lcsh.png' -m2

%% 
%plot effective radius
plot_geodesics(results);
export_fig 'lcsh-geodesics.png' -m2
